# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class Instance(_base.Resource):
    """DMS Instance resource"""
    resources_key = 'instances'
    base_path = '/instances'

    _query_mapping = resource.QueryParameters(
        'engine_name', 'name', 'status', 'include_failure',
        'exact_match_name',
        engine_name='engine',
        include_failure='includeFailure',
        exact_match_name='exactMatchName'
    )

    # capabilities
    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_delete = True
    allow_commit = True

    #: Properties
    #: The username of an instance.
    access_user = resource.Body('access_user')
    #: List of availability zones the instance belongs to
    availability_zones = resource.Body('available_zones', type=list)
    #: Billing mode
    charging_mode = resource.Body('charging_mode', type=int)
    #: IP address of the instance
    connect_address = resource.Body('connect_address')
    #: Instance creation time
    created_at = resource.Body('created_at')
    #: Instance description
    description = resource.Body('description')
    #: Message engine.
    engine_name = resource.Body('engine')
    #: Engine version
    engine_version = resource.Body('engine_version')
    #: Instance id
    instance_id = resource.Body('instance_id', alternate_id=True)
    #: Associate floating IP
    is_public = resource.Body('enable_publicip', type=bool)
    #: Enable SSL
    is_ssl = resource.Body('ssl_enable', type=bool)
    #: Kafka public status
    kafka_public_status = resource.Body('kafka_public_status')
    #: End time of the maintenance window
    maintenance_end = resource.Body('maintain_end')
    #: Beginning of the maintenance window
    maintenance_start = resource.Body('maintain_begin')
    #: Maximum number of partitions
    max_partitions = resource.Body('partition_num', type=int)
    #: Neutron network ID
    network_id = resource.Body('subnet_id')
    #: User password
    password = resource.Body('password')
    #: Port number of the instance
    port = resource.Body('port', type=int)
    #: Product ID
    product_id = resource.Body('product_id')
    #: Bandwidth of the public access
    public_bandwidth = resource.Body('public_bandwidth', type=int)
    #: Retention policy
    retention_policy = resource.Body('retention_policy')
    #: Router ID
    router_id = resource.Body('vpc_id')
    #: Router name
    router_name = resource.Body('vpc_name')
    #: Security group ID
    security_group_id = resource.Body('security_group_id')
    #: Security group Name
    security_group_name = resource.Body('security_group_name')
    #: Service type
    service_type = resource.Body('service_type')
    #: specification of the instance
    spec = resource.Body('specification')
    #: Specification code of the instance:
    #:  `dms.instance.kafka.cluster.c3.min`
    #:  `dms.instance.kafka.cluster.c3.small.2`
    #:  `dms.instance.kafka.cluster.c3.middle.2`
    #:  `dms.instance.kafka.cluster.c3.high.2`
    spec_code = resource.Body('resource_spec_code')
    #: Instance status
    #: CREATING, CREATEFAILED, RUNNING, ERROR, STARTING,
    #: RESTARTING, CLOSING, FROZEN
    status = resource.Body('status')
    #: Storage resource ID
    storage_resource_id = resource.Body('storage_resource_id')
    #: Storage I/O specification code
    storage_spec_code = resource.Body('storage_spec_code')
    #: Storage type
    storage_type = resource.Body('storage_type')
    #: Storage space GB
    storage = resource.Body('storage_space', type=int)
    #: Total storage space GB
    total_storage = resource.Body('total_storage_space', type=int)
    #: Instance type
    type = resource.Body('type')
    #: Used storage GB
    used_storage = resource.Body('used_storage_space', type=int)
    #: User ID
    user_id = resource.Body('user_id')
    #: User name
    user_name = resource.Body('user_name')

    def _action(self, session, action, id_list):
        body = {
            'action': action,
            'instances': id_list
        }

        response = session.post(
            '/instances/action',
            json=body
        )

        exceptions.raise_from_response(response)

        return

    def restart(self, session):
        """Restart specified instances
        """
        return self._action(session, 'restart', [self.id])

    def restart_batch(self, session, id_list):
        return self._action(session, 'restart', id_list)

    def delete_batch(self, session, id_list):
        """Delete batch of instances
        """
        return self._action(session, 'delete', id_list)

    def delete_failed(self, session):
        body = {
            'action': 'delete',
            'allFailure': 'kafka'
        }

        response = session.post(
            '/instances/action',
            json=body
        )
        exceptions.raise_from_response(response)
        return



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import openstack

from otcextensions.tests.functional import base


class TestOtcextensionsInitialization(base.BaseFunctionalTest):

    def test_basic(self):
        conn = openstack.connect(
            vendor_hook='otcextensions.sdk:load'
        )

        self.assertTrue(hasattr(conn.dns, 'add_router_to_zone'))
        list(conn.volume_backup.backup_policies())
        list(conn.obs.containers())



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
from enum import Enum

# from openstackclient.i18n import _


class ObsError(Enum):
    ERROR_INTERNAL = (1, "Internal error")
    ERROR_S3_SOURCE_MISSING = (404, 'Requested Object with key %s not found')
    ERROR_SOURCE_NOT_AVAILABLE = (123, 'Source %s is not available')

    def __init__(self, code, message, **kwargs):
        self.code = code
        self.message = message

    def get_formatted(self, args):
        self.message = self.message % args
        return self


class ObsException(Exception):
    def __init__(self, obs_error):
        self.code = obs_error.code
        self.message = obs_error.message



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from otcextensions.osclient.dms.v1 import queue
from otcextensions.tests.unit.osclient.dms.v1 import fakes


class TestDMSQueue(fakes.TestDMS):

    def setUp(self):
        super(TestDMSQueue, self).setUp()
        self.client = self.app.client_manager.dms


class TestListDMSQueue(TestDMSQueue):

    queues = fakes.FakeQueue.create_multiple(3)

    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    data = []

    for s in queues:
        data.append((
            s.id,
            s.name,
            s.queue_mode,
            s.description,
            s.redrive_policy,
            s.max_consume_count,
            s.retention_hours
        ))

    def setUp(self):
        super(TestListDMSQueue, self).setUp()

        self.cmd = queue.ListDMSQueue(self.app, None)

        self.client.queues = mock.Mock()

    def test_list_queue(self):
        arglist = [
        ]

        verifylist = [
            # ('group', None),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.queues.side_effect = [
            self.queues
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.queues.assert_called_once_with()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowDMSQueue(TestDMSQueue):

    _data = fakes.FakeQueue.create_one()

    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    data = (
        _data.id,
        _data.name,
        _data.queue_mode,
        _data.description,
        _data.redrive_policy,
        _data.max_consume_count,
        _data.retention_hours
    )

    def setUp(self):
        super(TestShowDMSQueue, self).setUp()

        self.cmd = queue.ShowDMSQueue(self.app, None)

        self.client.show_queue = mock.Mock()

    def test_show_default(self):
        arglist = [
            'test_queue'
        ]
        verifylist = [
            ('queue', 'test_queue')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_queue.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_queue.assert_called()

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteDMSQueue(TestDMSQueue):

    def setUp(self):
        super(TestDeleteDMSQueue, self).setUp()

        self.cmd = queue.DeleteDMSQueue(self.app, None)

        self.client.delete_queue = mock.Mock()

    def test_delete(self):
        arglist = ['t1']
        verifylist = [
            ('queue', ['t1'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_queue.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call('t1')]

        self.client.delete_queue.assert_has_calls(calls)
        self.assertEqual(1, self.client.delete_queue.call_count)

    def test_delete_multiple(self):
        arglist = [
            't1',
            't2',
        ]
        verifylist = [
            ('queue', ['t1', 't2'])
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_queue.side_effect = [{}, {}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        calls = [mock.call('t1'), mock.call('t2')]

        self.client.delete_queue.assert_has_calls(calls)
        self.assertEqual(2, self.client.delete_queue.call_count)


class TestCreateDMSQueue(TestDMSQueue):

    _data = fakes.FakeQueue.create_one()

    columns = ('ID', 'name', 'queue_mode', 'description', 'redrive_policy',
               'max_consume_count', 'retention_hours')

    data = (
        _data.id,
        _data.name,
        _data.queue_mode,
        _data.description,
        _data.redrive_policy,
        _data.max_consume_count,
        _data.retention_hours
    )

    def setUp(self):
        super(TestCreateDMSQueue, self).setUp()

        self.cmd = queue.CreateDMSQueue(self.app, None)

        self.client.create_queue = mock.Mock()

    def test_show_default(self):
        arglist = [
            'name',
            'NORMAL',
            '--description', 'descr',
            '--redrive_policy', 'enable',
            '--max_consume_count', '1',
            '--retention_hours', '2'
        ]
        verifylist = [
            ('name', 'name'),
            ('queue_mode', 'NORMAL'),
            ('description', 'descr'),
            ('redrive_policy', 'enable'),
            ('max_consume_count', 1),
            ('retention_hours', 2)
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_queue.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_queue.assert_called_with(
            description='descr',
            max_consume_count=1,
            name='name',
            queue_mode='NORMAL',
            redrive_policy='enable',
            retention_hours=2
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from openstack import utils
from otcextensions.sdk.imsv1.v1 import async_job as _async_job
from otcextensions.sdk.imsv1.v1 import image_export as _image_export


class Proxy(proxy.Proxy):
    skip_discovery = True

    def get_async_job(self, job_id):
        """Get an asynchronous job

        :returns: One
             :class:`~otcextensions.sdk.imsv2.v1.async_job.AsyncJob`
        """
        attrs = {
            'id': job_id
        }
        base_path = '{project_id}/jobs'.format(
            project_id=self.session.get_project_id()
        )
        return self._get(_async_job.AsyncJob, base_path=base_path, **attrs)

    def wait_for_async_job(self, job_id, timeout=600,
                           message="Timeout waiting for asynchronous job.",
                           wait=10):
        """Wait for an asynchronous job to finish

        :returns: One
             :class:`~otcextensions.sdk.imsv2.v1.async_job.AsyncJob`
        """
        # Maybe someone provided an object instead of a job_id
        # try to get the job id from that object if possible
        if hasattr(job_id, 'job_id'):
            job_id = job_id.job_id

        for count in utils.iterate_timeout(
                timeout=timeout,
                message=message,
                wait=wait):
            job = self.get_async_job(job_id)
            if job.status == "FAIL":
                raise RuntimeError(
                    "Asynchronous job {job_id} failed.".format(job_id=job_id)
                )
            elif job.status not in ['INIT', 'RUNNING']:
                break
        return job

    def export_image(self, **attrs):
        """Export an image

        :returns: One
             :class:`~otcextensions.sdk.imsv1.v1.image_export.ImageExport`
        """
        return self._create(_image_export.ImageExport, **attrs)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DCS Restore Record v1 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {
    }
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListRestoreRecords(command.Lister):
    _description = _('List restore records of a single DCS instance')
    columns = ('id', 'name', 'backup_id', 'progress', 'status', 'error_code')

    def get_parser(self, prog_name):
        parser = super(ListRestoreRecords, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or ID of the instance')
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of records to return.')
        )
        parser.add_argument(
            '--start',
            metavar='<start>',
            type=int,
            help=_('Start number for querying.')
        )
        parser.add_argument(
            '--from_time',
            metavar='<yyyyMMddHHmmss>',
            help=_('Start time of the period to be queried.')
        )
        parser.add_argument(
            '--to_time',
            metavar='<yyyyMMddHHmmss>',
            help=_('End time of the period to be queried.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dcs

        query = {}
        if parsed_args.limit:
            query['limit'] = parsed_args.limit
        if parsed_args.start:
            query['start'] = parsed_args.start
        if parsed_args.from_time:
            query['begin_time'] = parsed_args.from_time
        if parsed_args.to_time:
            query['end_time'] = parsed_args.to_time

        inst = client.find_instance(name_or_id=parsed_args.instance,
                                    ignore_missing=False)
        data = client.restore_records(
            instance={'id': inst.id},
            **query
        )

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns,
                 ) for s in data))
        return table


class RestoreBackup(command.ShowOne):
    _description = _('Restore a backup of a DCS instance')

    def get_parser(self, prog_name):
        parser = super(RestoreBackup, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Name or ID of the DCS instance to take backup from.')
        )
        parser.add_argument(
            '--backup',
            metavar='<id>',
            required=True,
            help=_('ID of the backup to restore.')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the restore record.')
        )
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dcs

        attrs = {}

        if parsed_args.description:
            attrs['description'] = parsed_args.description

        inst = client.find_instance(name_or_id=parsed_args.instance,
                                    ignore_missing=False)

        obj = client.restore_instance(
            instance={'id': inst.id},
            backup_id=parsed_args.backup,
            **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from otcextensions.sdk.tms.v1 import predefined_tag as _predefined_tag
from otcextensions.sdk.tms.v1 import resource_tag as _resource_tag
from urllib.parse import urlparse


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _get_endpoint_with_api_version(self, api_version):
        url_parts = urlparse(self.get_endpoint())
        alternate_endpoint = '{scheme}://{netloc}/{api_version}'.format(
            scheme=url_parts.scheme,
            netloc=url_parts.netloc,
            api_version=api_version
        )
        return alternate_endpoint

    # ======== Predefined Tag ========
    def predefined_tags(self, **query):
        """Retrieve a generator of PredefinedTags

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.
            * `key`: Tag key
            * `value`: Tag value
            * `limit`: Number of query records
            * `marker`: Paging location identifier
            * `order_field`: Field for sorting
            * `order_method`: Sorting will be asc or desc


        :returns: A generator of backup
            :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
            instances
        """
        return self._list(_predefined_tag.PredefinedTag, **query)

    def create_predefined_tag(self, **attrs):
        """Create a new predefined tag with attrs

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
        """
        tag = _predefined_tag.PredefinedTag()
        return tag.add_tag(self, **attrs)

    def delete_predefined_tag(self, **attrs):
        """Delete a new predefined tag with attrs

        :param dict attrs: Keyword arguments which will be used to delete a
            :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
        """
        tag = _predefined_tag.PredefinedTag()
        return tag.delete_tag(self, **attrs)

    def update_predefined_tag(self, **attrs):
        """Update a predefined tag

        :param dict attrs: The attributes to update on the predefined tag

        :returns: The updated tag.

        :rtype: :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
        """
        return self._update(_predefined_tag.PredefinedTag,
                            **attrs)

    # ======== Resource Tag ========
    def resource_tags(self, resource_id, resource_type, project_id=None):
        """Retrieve a generator of ResourceTags
        'project_id', 'resource_id', 'resource_type'

        :param project_id: Optional ID of the project
        :param resource_type: Mandatory type of the resource
        :param resource_id: Mandatory id of the resource

        :returns: A generator of backup
            :class:`~otcextensions.sdk.tms.v1.resource_tag.ResourceTag`
            instances
        """
        base = self._get_endpoint_with_api_version('v2.0')
        base_path = (f"{base}/resources/{resource_id}/tags?project_id="
                     f"{project_id}&resource_type={resource_type}")
        return self._list(_resource_tag.ResourceTag, base_path=base_path)

    def create_resource_tag(self, **attrs):
        """Create a new resource tag with attrs

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
        """
        return self._create(_resource_tag.ResourceTag, **attrs)

    def delete_resource_tag(self, **attrs):
        """Delete a new resource tag with attrs

        :param dict attrs: Keyword arguments which will be used to delete a
            :class:`~otcextensions.sdk.tms.v1.predefined_tag.PredefinedTag`
        """
        return self._delete(_resource_tag.ResourceTag, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from otcextensions.tests.functional.sdk.apig import TestApiG


class TestGwFeatures(TestApiG):
    def setUp(self):
        super(TestGwFeatures, self).setUp()
        self.create_gateway()
        self.gateway_id = TestGwFeatures.gateway.id
        # self.gateway_id = "be76ca6de5fe4aa7af503c03b3b44dea"
        self.attrs = {
            "name": "route",
            "enable": True,
            "config": "{\"user_routes\":[\"172.16.128.0/20\","
                      "\"172.16.0.0/20\"]}",
        }
        self.feat = self.client.configure_gateway_feature(
            gateway=self.gateway_id,
            **self.attrs
        )

        self.disable_attrs = {
            "name": "route",
            "enable": False,
            "config": "{\"user_routes\":[]}",
        }
        self.addCleanup(
            self.client.configure_gateway_feature,
            gateway=self.gateway_id,
            **self.disable_attrs
        )
        self.addCleanup(self.delete_gateway())

    def test_list_gateway_features(self):
        features = list(self.client.gateway_features(
            gateway=self.gateway_id))
        self.assertGreaterEqual(len(features), 10)

    def test_list_available_gateway_features(self):
        features = list(self.client.supported_gateway_features(
            gateway=self.gateway_id, limit=500))
        self.assertGreaterEqual(len(features), 1)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class ExportApi(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/openapi/export'

    _query_mapping = resource.QueryParameters('oas_version')

    gateway_id = resource.URI('gateway_id')
    env_id = resource.Body('env_id')
    group_id = resource.Body('group_id')
    define = resource.Body('define')
    type = resource.Body('type')
    version = resource.Body('version')
    apis = resource.Body('apis', type=list)

    def _export_api(self, session, gateway_id,
                    full_path, **attrs):
        uri = f'apigw/instances/{gateway_id}/openapi/export'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        file_type = attrs['type'] if 'type' in attrs else 'json'
        with open(f'{full_path}.{file_type}', 'wb') as f:
            f.write(response.content)
        return None


class SuccessSpec(resource.Resource):
    id = resource.Body('id')
    method = resource.Body('method')
    path = resource.Body('path')
    action = resource.Body('action')


class FailureSpec(resource.Resource):
    method = resource.Body('method')
    path = resource.Body('path')
    error_code = resource.Body('error_code')
    error_msg = resource.Body('error_msg')


class SwaggerSpec(resource.Resource):
    id = resource.Body('id')
    result = resource.Body('result')


class IgnoreSpec(resource.Resource):
    method = resource.Body('method')
    path = resource.Body('path')


class ImportApi(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/openapi/import'

    gateway_id = resource.URI('gateway_id')
    is_create_group = resource.Body('is_create_group', type=bool)
    group_id = resource.Body('group_id')
    extend_mode = resource.Body('extend_mode')
    simple_mode = resource.Body('simple_mode', type=bool)
    mock_mode = resource.Body('mock_mode', type=bool)
    api_mode = resource.Body('api_mode')
    file_name = resource.Body('file_name')

    success = resource.Body('success', type=list, list_type=SuccessSpec)
    failure = resource.Body('failure', type=list, list_type=FailureSpec)
    swagger = resource.Body('swagger', type=SwaggerSpec)
    ignore = resource.Body('ignore', type=list, list_type=IgnoreSpec)

    def _import_api(self, session, gateway_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/openapi/import'
        response = session.post(uri, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource


class JobFlavor(resource.Resource):
    base_path = "/job/resource-specs"

    resources_key = "specs"

    # capabilities
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "engine_id",
        "job_type",
        "project_type",
    )

    # Properties
    #: Number of cores of the resource specifications.
    core = resource.Body("core")
    #: CPU memory of the resource specifications.
    cpu = resource.Body("cpu")
    #: Number of GPUs of the resource specifications.
    gpu_num = resource.Body("gpu_num", type=int)
    #: GPU type of the resource specifications.
    gpu_type = resource.Body("gpu_type")
    #: Interface type.
    interface_type = resource.Body("interface_type", type=int)
    #: Maximum number of nodes that can be selected.
    max_num = resource.Body("max_num", type=int)
    #: Whether the resources of the selected specifications are sufficient.
    no_resource = resource.Body("no_resource", type=bool)
    #: Type of the resource specifications.
    spec_code = resource.Body("spec_code")
    #: ID of the resource specifications.
    spec_id = resource.Body("spec_id", type=int)
    #: SSD size of a resource flavor.
    storage = resource.Body("storage")
    #: Number of pricing units.
    unit_num = resource.Body("unit_num", type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.rds.v1 import flavor

# RDS requires those headers to be present in the request, to native API
# otherwise 404
RDS_HEADERS = {
    'Content-Type': 'application/json',
    'X-Language': 'en-us'
}

# RDS requires those headers to be present in the request, to OS-compat API
# otherwise 404
OS_HEADERS = {
    'Content-Type': 'application/json',
}

IDENTIFIER = 'IDENTIFIER'
EXAMPLE = {
    # 'id': IDENTIFIER,
    'links': '1',
    'name': '2',
    'ram': 3,
    'str_id': 'some-str-id',
    # 'project_id': PROJECT_ID,
}


class TestFlavor(base.TestCase):

    def setUp(self):
        super(TestFlavor, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sot = flavor.Flavor.existing(**EXAMPLE)

    def test_basic(self):
        sot = flavor.Flavor()
        self.assertEqual('flavor', sot.resource_key)
        self.assertEqual('flavors', sot.resources_key)
        self.assertEqual('/flavors', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_get)
        self.assertFalse(sot.allow_update)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        sot = flavor.Flavor.existing(**EXAMPLE)
        # self.assertEqual(EXAMPLE['str_id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['ram'], sot.ram)

    def test_list(self):

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {'flavors': [EXAMPLE]}

        self.sess.get.return_value = mock_response

        result = list(self.sot.list(self.sess))

        self.sess.get.assert_called_once_with(
            '/flavors',
            params={},
        )

        self.assertEqual([flavor.Flavor(**EXAMPLE)], result)

    def test_get(self):

        sot = flavor.Flavor.existing(id=123)
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}

        res_json = {
            "ram": 2048,
            "id": 1,
            "links": None,
            "name": "rds.pg.c2.medium",
            "specCode": "rds.pg.c2.medium",
        }

        # Sadly res_json is deleted somewhere in __GET__, so
        # pass a copy of it
        mock_response.json.return_value = copy.deepcopy(res_json)

        self.sess.get.return_value = mock_response

        res = sot.get(self.sess)

        self.sess.get.assert_called_once_with(
            'flavors/123',
        )

        self.assertEqual(2048, res.ram)
        self.assertEqual(res_json['name'], res.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource
from openstack import utils


class Resource(resource.Resource):

    resources_key = 'tags'
    resource_key = 'tag'

    # capabilities
    allow_create = True
    allow_delete = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        'key'
    )

    # Properties
    #: Specifies the tag key
    key = resource.Body('key')
    #: Specifies the tag value
    value = resource.Body('value')

    def _prepare_request(self, requires_id=None, prepend_key=False,
                         patch=False, base_path=None, params=None, **kwargs):
        """Prepare a request to be sent to the server

        Create operations don't require an ID, but all others do,
        so only try to append an ID when it's needed with
        requires_id. Create and update operations sometimes require
        their bodies to be contained within an dict -- if the
        instance contains a resource_key and prepend_key=True,
        the body will be wrapped in a dict with that key.
        If patch=True, a JSON patch is prepared instead of the full body.

        Return a _Request object that contains the constructed URI
        as well a body and headers that are ready to send.
        Only dirty body and header contents will be returned.
        """
        if requires_id is None:
            requires_id = self.requires_id

        body = self._prepare_request_body(patch, prepend_key)
        headers = {}

        if base_path is None:
            base_path = self.base_path
        uri = base_path % self._uri.attributes
        if requires_id:
            if self.id is None:
                raise exceptions.InvalidRequest(
                    "Request requires an ID but none was found")

            uri = utils.urljoin(uri, self.id)

        uri = utils.urljoin(self.location.project['id'], uri)

        return resource._Request(uri, body, headers)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class Product(_base.Resource):
    """DMS Product resource"""
    base_path = '/products'
    resources_key = 'Hourly'

    # capabilities
    allow_list = True

    #: Properties
    engine_name = resource.Body('engine_name')
    engine_version = resource.Body('engine_version')
    #: Indicates the maximum number of messages per unit time.
    tps = resource.Body('tps')
    #: Indicates the message storage space.
    storage = resource.Body('storage')
    #: Indicates the maximum number of topics in a Kafka instance.
    partition_num = resource.Body('partition_num')
    #: Indicates the product ID.
    product_id = resource.Body('product_id')
    #: Indicates the specification ID.
    spec_code = resource.Body('spec_code')
    #: Indicates the I/O information.
    io = resource.Body('io', type=list, list_type=dict)
    #: Indicates the bandwidth of a Kafka instance.
    bandwidth = resource.Body('bandwidth')
    #: Indicates AZs where there are available resources.
    availability_zones = resource.Body('available_zones', type=list)
    #: Indicated AZs where it is not available
    unavailable_zones = resource.Body('unavailable_zones', type=list)
    #: Indicates the VM specifications of the instance resources.
    vm_flavor_id = resource.Body('ecs_flavor_id')
    #: Indicates the instance architecture type.
    #: Currently, only x86 is supported.
    arch_type = resource.Body('arch_type')

    @classmethod
    def list(cls, session, **params):
        # This API is a total disaster (b*****it)
        # Show me who was so smart to develop it...
        session = cls._get_session(session)

        base_path = cls.base_path
        params.pop('paginated', None)
        params.pop('base_path', None)
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=False)
        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        # Copy query_params due to weird mock unittest interactions
        response = session.get(
            uri,
            endpoint_override=session.endpoint_override.replace(
                '%(project_id)s', ''),
            headers={"Accept": "application/json"},
            params=query_params.copy())
        exceptions.raise_from_response(response)
        data = response.json()

        for k, v in data.items():
            for rec in v:
                engine = rec
                engine_name = engine.get('name')
                engine_version = engine.get('version')
                for madness in engine.get('values', []):
                    for spec in madness.get('detail', []):
                        value = cls.existing(
                            connection=session._get_connection(),
                            engine_name=engine_name,
                            engine_version=engine_version,
                            **spec)
                        yield value



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import random
import uuid

import mock

from otcextensions.sdk.cce.v3 import cluster
from otcextensions.sdk.cce.v3 import cluster_node
from otcextensions.sdk.cce.v3 import node_pool
from otcextensions.sdk.cce.v3 import cluster_cert
from otcextensions.tests.unit.osclient import test_base


class TestCCE(test_base.TestCommand):

    def setUp(self):
        super(TestCCE, self).setUp()

        self.app.client_manager.cce = mock.Mock()
        self.app.client_manager.sdk_connection = mock.Mock()
        self.client = self.app.client_manager.cce
        self.sdk_client = self.app.client_manager.sdk_connection


class FakeCluster(test_base.Fake):
    """Fake one or more Cluster"""

    @classmethod
    def generate(cls):
        object_info = {
            'kind': 'Cluster',
            'metadata': {
                'uid': 'id-' + uuid.uuid4().hex,
                'name': 'name-' + uuid.uuid4().hex,
            },
            'spec': {
                'type': random.choice(['VirtualMachine', 'BareMetal']),
                'flavor': uuid.uuid4().hex,
                'version': uuid.uuid4().hex,
                'host_network': {
                    'vpc': 'vpc-' + uuid.uuid4().hex,
                    'subnet': 'subnet-' + uuid.uuid4().hex,
                },
                'container_network': {
                    'mode': random.choice(['overlay_l2']),
                }
            },
            'status': {
                'phase': 'Available',
                'endpoints': {
                    'internal': uuid.uuid4().hex,
                    'external_otc': uuid.uuid4().hex,
                },
                'jobID': uuid.uuid4().hex
            }
        }
        obj = cluster.Cluster.existing(**object_info)
        return obj


class FakeClusterNode(test_base.Fake):
    """Fake one or more Cluster node"""

    @classmethod
    def generate(cls):
        object_info = {
            'kind': 'Node',
            'metadata': {
                'uid': 'id-' + uuid.uuid4().hex,
                'name': 'name-' + uuid.uuid4().hex,
            },
            'spec': {
                'flavor': 'flavor-' + uuid.uuid4().hex,
                'availability_zone': 'az-' + uuid.uuid4().hex,
                'os': 'os-' + uuid.uuid4().hex,
                'login': {
                    'sshKey': 'key-' + uuid.uuid4().hex,
                },
                'rootVolume': {
                    'type': 'SATA',
                    'size': random.randint(40, 100)
                },
                'dataVolumes': [
                    {
                        'type': 'SSD',
                        'size': random.randint(1, 15000),
                    },
                    {
                        'type': 'SAS',
                        'size': random.randint(1, 15000),
                    },
                ],
            },
            'status': {
                'phase': 'Available',
                'server_id': 'sid-' + uuid.uuid4().hex,
                'floating_ip': 'fip-' + uuid.uuid4().hex,
                'private_ip': 'pip-' + uuid.uuid4().hex,
            },
        }
        obj = cluster_node.ClusterNode.existing(**object_info)
        return obj


class FakeNodePool(test_base.Fake):
    """Fake one or more CCE Nodepools"""

    @classmethod
    def generate(cls):
        object_info = {
            'kind': 'NodePool',
            'apiVersion': 'v3',
            'metadata': {
                'name': 'name-' + uuid.uuid4().hex
            },
            'spec': {
                'initialNodeCount': 0,
                'type': 'vm',
                'autoscaling': {
                    'enable': random.choice([True, False]),
                    'minNodeCount': 0,
                    'maxNodeCount': random.randint(0, 100),
                    'scaleDownCooldownTime': random.randint(1, 10),
                    'priority': random.randint(1, 99)
                },
                'nodeManagement': {
                    'serverGroupReference': 'sg-' + uuid.uuid4().hex
                },
                'nodeTemplate': {
                    'flavor': 's2.large.2 ',
                    'az': random.choice([
                        'random',
                        'eu-de-01',
                        'eu-de-02',
                        'eu-de-03']),
                    'os': 'EulerOS 2.5',
                    'login': {
                        'sshKey': 'key-' + uuid.uuid4().hex
                    },
                    'rootVolume': {
                        'volumetype': random.choice(['SAS', 'SATA', 'SSD']),
                        'size': random.randint(40, 32768),
                    },
                    'dataVolumes': [
                        {
                            'volumetype': random.choice([
                                'SAS', 'SATA', 'SSD']),
                            'size': random.randint(100, 32768),
                            'extendParam': {
                                'useType': 'docker'
                            }
                        }
                    ],
                    'billingMode': 0,
                    'extendParam': {
                        'alpha.cce/preInstall': 'bHMgLWw=',
                        'alpha.cce/postInstall': 'bHMgLWwK',
                        'maxPods': 110,
                    },
                    'k8sTags': {
                        't1-' + uuid.uuid4().hex: 'v1-' + uuid.uuid4().hex,
                        't2-' + uuid.uuid4().hex: 'v2-' + uuid.uuid4().hex,
                    },
                    'taints': [
                        {
                            'key': 'key-' + uuid.uuid4().hex,
                            'value': 'value-' + uuid.uuid4().hex,
                            'effect': random.choice([
                                'NoSchedule',
                                'PrefereNoSchedule',
                                'NoExecute'])
                        },
                        {
                            'key': 'key-' + uuid.uuid4().hex,
                            'value': 'value-' + uuid.uuid4().hex,
                            'effect': random.choice([
                                'NoSchedule',
                                'PrefereNoSchedule',
                                'NoExecute'])
                        }
                    ],
                    'userTags': [
                        {
                            'key': 'key-' + uuid.uuid4().hex,
                            'value': 'value-' + uuid.uuid4().hex,
                        },
                        {
                            'key': 'key-' + uuid.uuid4().hex,
                            'value': 'value-' + uuid.uuid4().hex,
                        }
                    ],
                    'nodeNicSpec': {
                        'primaryNic': {
                            'subnetId': 'nw-' + uuid.uuid4().hex,
                        }
                    }
                }
            },
            'status': {
                'currentNode': random.randint(0, 100),
                'phase': ''
            }
        }
        obj = node_pool.NodePool.existing(**object_info)
        return obj


class FakeClusterCertificate(test_base.Fake):
    """Fake one or more Cluster Certificate"""

    @classmethod
    def generate(cls):
        object_info = {
            'ca': uuid.uuid4().hex,
            'client_certificate': uuid.uuid4().hex,
            'client_key': uuid.uuid4().hex,
            'context': {
                'name': uuid.uuid4().hex[:8],
                'user': uuid.uuid4().hex[:8],
                'cluster': uuid.uuid4().hex[:8],
            },
        }
        obj = cluster_cert.ClusterCertificate.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from otcextensions.sdk.modelartsv1.v1.service import ConfigSpec


class ResultSpec(resource.Resource):
    #: Node ID.
    node_id = resource.Body("node_id")
    #: Node name.
    node_name = resource.Body("node_name")
    #: Operation type.
    operation = resource.Body("operation")
    #: Operation result.
    result = resource.Body("result", type=bool)


class ServiceLog(resource.Resource):
    base_path = "/services/%(service_id)s/logs"

    resources_key = "logs"

    allow_list = True

    _query_mapping = resource.QueryParameters(
        "update_time",
    )

    # Properties
    #: ID of a dedicated resource pool.
    cluster_id = resource.Body("cluster_id")
    #: Updated service configurations.
    config = resource.Body("config", type=list, list_type=ConfigSpec)
    #: Personalized configuration.
    extend_config = resource.Body("extend_config", type=list)
    #: Number of nodes that fail to be operated. This parameter is
    #:  returned when infer_type is set to edge.
    failed_num = resource.Body("failed_num", type=int)
    #: Update result. The value can be `SUCCESS`, `FAIL`, or `RUNNING`.
    result = resource.Body("result")
    #: Operation result details. This parameter is returned when
    #:  infer_type is set to edge.
    result_detail = resource.Body(
        "result_detail", type=list, list_type=ResultSpec
    )
    #: Service ID.
    service_id = resource.URI("service_id")
    #: Number of nodes that are successfully operated. This parameter
    #:  is returned when infer_type is set to edge.
    success_num = resource.Body("success_num", type=int)
    #: Time when a service is updated, in milliseconds calculated
    #:  from 1970.1.1 0:0:0 UTC.
    update_time = resource.Body("update_time", type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.modelartsv1.v1 import job_engine
from otcextensions.tests.unit.osclient.modelartsv1.v1 import fakes


class TestListJobEngines(fakes.TestModelartsv1):
    objects = fakes.FakeJobEngine.create_multiple(3)

    column_list_headers = (
        "Engine Id",
        "Engine Name",
        "Engine Type",
        "Engine Version",
    )

    data = []

    for s in objects:
        data.append(
            (
                s.engine_id,
                s.engine_name,
                s.engine_type,
                s.engine_version,
            )
        )

    def setUp(self):
        super(TestListJobEngines, self).setUp()

        self.cmd = job_engine.ListJobEngines(self.app, None)

        self.client.job_engines = mock.Mock()
        self.client.api_mock = self.client.job_engines

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            "--job-type",
            "inference",
        ]

        verifylist = [
            ("job_type", "inference"),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            job_type="inference",
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.common import format
from otcextensions.sdk.anti_ddos.v1 import status

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE_STATUS = {
    "status": "enable",
}

EXAMPLE_TASK = {
    "task_status": "running",
    "task_msg": "",
}

EXAMPLE_LOG = {
    "start_time": 1473217200000,
    "end_time": 1473242400000,
    "status": 1,
    "trigger_bps": 51106,
    "trigger_pps": 2600,
    "trigger_http_pps": 3589
}

EXAMPLE_DAY_STAT = {
    "period_start": 1472713370609,
    "bps_in": 0,
    "bps_attack": 0,
    "total_bps": 0,
    "pps_in": 0,
    "pps_attack": 0,
    "total_pps": 0
}

EXAMPLE_WEEK_DATA_STAT = {
    "ddos_intercept_times": 0,
    "ddos_blackhole_times": 0,
    "max_attack_bps": 0,
    "max_attack_conns": 0,
    "period_start_date": 1474214461651
}

EXAMPLE_WEEK_STAT = {
    "ddos_intercept_times": 0,
    "weekdata": [EXAMPLE_WEEK_DATA_STAT],
    "top10": [
        {
            "floating_ip_address": "192.168.44.69",
            "times": 23
        }
    ]
}


class TestStatus(base.TestCase):

    def test_basic(self):
        sot = status.FloatingIPStatus()

        self.assertEqual('/antiddos/%(floating_ip_id)s/status', sot.base_path)

        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):

        sot = status.FloatingIPStatus(floating_ip_id=FAKE_ID, **EXAMPLE_STATUS)
        self.assertEqual(FAKE_ID, sot.floating_ip_id)
        self.assertEqual(EXAMPLE_STATUS['status'], sot.status)


class TestTask(base.TestCase):

    def test_basic(self):
        sot = status.TaskStatus()

        self.assertEqual('/query_task_status', sot.base_path)

        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):

        sot = status.TaskStatus(**EXAMPLE_TASK)
        self.assertEqual(EXAMPLE_TASK['task_status'], sot.task_status)
        self.assertEqual(EXAMPLE_TASK['task_msg'], sot.task_msg)


class TestLog(base.TestCase):

    def test_basic(self):
        sot = status.FloatingIPEvent()

        self.assertEqual('/antiddos/%(floating_ip_id)s/logs', sot.base_path)
        self.assertEqual('logs', sot.resources_key)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = status.FloatingIPEvent(**EXAMPLE_LOG)
        self.assertEqual(
            format.TimeTMsStr().deserialize(EXAMPLE_LOG['start_time']),
            sot.start_time),
        self.assertEqual(
            format.TimeTMsStr().deserialize(EXAMPLE_LOG['end_time']),
            sot.end_time),
        self.assertEqual(EXAMPLE_LOG['status'], sot.status)
        self.assertEqual(EXAMPLE_LOG['trigger_bps'], sot.trigger_bps)
        self.assertEqual(EXAMPLE_LOG['trigger_pps'], sot.trigger_pps)
        self.assertEqual(EXAMPLE_LOG['trigger_http_pps'],
                         sot.trigger_http_pps)


class TestStatDay(base.TestCase):

    def test_basic(self):
        sot = status.FloatingIPDayStat()

        self.assertEqual('/antiddos/%(floating_ip_id)s/daily', sot.base_path)
        self.assertEqual('data', sot.resources_key)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = status.FloatingIPDayStat(**EXAMPLE_DAY_STAT)
        self.assertEqual(
            format.TimeTMsStr().deserialize(EXAMPLE_DAY_STAT['period_start']),
            sot.period_start),
        self.assertEqual(EXAMPLE_DAY_STAT['bps_in'], sot.bps_in)
        self.assertEqual(EXAMPLE_DAY_STAT['bps_attack'], sot.bps_attack)
        self.assertEqual(EXAMPLE_DAY_STAT['total_bps'], sot.total_bps)
        self.assertEqual(EXAMPLE_DAY_STAT['pps_in'], sot.pps_in)
        self.assertEqual(EXAMPLE_DAY_STAT['pps_attack'], sot.pps_attack)
        self.assertEqual(EXAMPLE_DAY_STAT['total_pps'], sot.total_pps)


class TestStatWeek(base.TestCase):

    def test_basic(self):
        sot = status.FloatingIPWeekStat()

        self.assertEqual('/antiddos/weekly', sot.base_path)

        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):

        sot = status.FloatingIPWeekStat(**EXAMPLE_WEEK_STAT)
        self.assertEqual(EXAMPLE_WEEK_STAT['ddos_intercept_times'],
                         sot.ddos_intercept_times)
        self.assertEqual(EXAMPLE_WEEK_STAT['top10'],
                         sot.top10)

    def test_make_data(self):

        sot = status.FloatingIPWeekStatData(**EXAMPLE_WEEK_DATA_STAT)
        self.assertEqual(
            format.TimeTMsStr().deserialize(
                EXAMPLE_WEEK_DATA_STAT['period_start_date']),
            sot.period_start_date),
        self.assertEqual(EXAMPLE_WEEK_DATA_STAT['ddos_intercept_times'],
                         sot.ddos_intercept_times)
        self.assertEqual(EXAMPLE_WEEK_DATA_STAT['ddos_blackhole_times'],
                         sot.ddos_blackhole_times)
        self.assertEqual(EXAMPLE_WEEK_DATA_STAT['max_attack_bps'],
                         sot.max_attack_bps)
        self.assertEqual(EXAMPLE_WEEK_DATA_STAT['max_attack_conns'],
                         sot.max_attack_conns)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class ResourcesSpec(resource.Resource):

    # Properties
    # quota type
    type = resource.Body('type')
    # total amount of quota
    quota = resource.Body('quota', type=int)
    # quota unit
    unit = resource.Body('unit')
    # used amount of quota
    used = resource.Body('used', type=int)


class Quota(resource.Resource):
    resources_key = 'quotas'
    base_path = '/quotas'

    # capabilities
    allow_list = True

    # Properties
    resources = resource.Body('resources', type=list,
                              list_type=ResourcesSpec)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import json

from openstackclient.tests.functional import base
from tempest.lib import exceptions


class TestRdsDatastore(base.TestCase):
    """Functional tests for RDS Datastore. """

    def test_datastore_list(self):
        json_output = json.loads(self.openstack(
            'rds datastore type list -f json '
        ))

        self.assertItemsEqual(
            ['postgresql', 'mysql', 'sqlserver'],
            [ds.get('Name', None).lower() for ds in json_output]
        )

    def test_datastore_version_list(self):
        datastores = json.loads(self.openstack(
            'rds datastore type list -f json'
        ))
        for datastore in datastores:
            json_output = json.loads(self.openstack(
                'rds datastore version list {ds} '
                '-f json'.format(ds=datastore['Name'])
            ))
            self.assertIsNotNone(json_output)
            self.assertEqual(
                ['ID', 'Name'],
                list(json_output[0].keys())
            )

    def test_invalid_datastore_version_list(self):
        self.assertRaises(
            exceptions.CommandFailed,
            self.openstack,
            'rds datastore version list invalid_ds'
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.imsv1.v1 import _proxy
from otcextensions.sdk.imsv1.v1 import async_job


class TestImsProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestImsProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestImsAsyncJob(TestImsProxy):

    def test_async_job_get(self):
        self._verify('openstack.proxy.Proxy._get',
                     self.proxy.get_async_job,
                     expected_args=[async_job.AsyncJob],
                     method_kwargs={
                         'job_id': '1'
                     },
                     expected_kwargs={
                         'id': '1'
                     })



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import ip_address_group

EXAMPLE = {
    'id': 'ip-address-id',
    'description': 'ip address group',
    'project_id': 'project-id',
    'name': 'ipgroup-name',
    'ip_list': [
        {
            'ip': '192.168.0.2',
            'description': 'ip description'
        }
    ],
    'listeners': [{'id': "8e92b7c3-cdae-4039-aa62-c76d09a5950a"}],
    'created_at': 'created-at',
    'updated_at': 'updated-at'
}


class TestIpAddressGroup(base.TestCase):

    def setUp(self):
        super(TestIpAddressGroup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.delete = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.retriable_status_codes = ()
        self.sot = ip_address_group.IpAddressGroup()

    def test_basic(self):
        sot = ip_address_group.IpAddressGroup()
        self.assertEqual('ipgroup', sot.resource_key)
        self.assertEqual('ipgroups', sot.resources_key)
        self.assertEqual('/elb/ipgroups', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = ip_address_group.IpAddressGroup.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(
            EXAMPLE['ip_list'][0]['ip'], sot.ip_list[0]['ip'])
        self.assertEqual(
            EXAMPLE['ip_list'][0]['description'],
            sot.ip_list[0]['description'])
        self.assertEqual(
            EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['listeners'], sot.listeners)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)

    def test_get(self):
        sot = ip_address_group.IpAddressGroup.existing(id=EXAMPLE['id'])
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {
            'ipgroup': EXAMPLE.copy()}

        self.sess.get.return_value = mock_response

        result = sot.fetch(self.sess)

        self.sess.get.assert_called_once_with(
            'elb/ipgroups/%s' %
            EXAMPLE['id'],
            microversion=None,
            params={},
            skip_cache=False
        )

        self.assertEqual(sot, result)
        self.assertEqual(EXAMPLE['id'], result.id)
        self.assertEqual(EXAMPLE['name'], result.name)

    def test_update_ip_addresses(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = ip_address_group.IpAddressGroup.existing(id=EXAMPLE['id'])

        sot.update_ip_addresses(self.sess, ip_list=[{'ip': '192.168.0.2',
                                                     'description': 'test'}])

        self.sess.post.assert_called_once_with(
            'elb/ipgroups/%s/iplist/create-or-update' % EXAMPLE['id'],
            json={'ipgroup': {'ip_list': [{'ip': '192.168.0.2',
                                           'description': 'test'}]}}
        )

    def test_delete_ip_addresses(self):
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot = ip_address_group.IpAddressGroup.existing(id=EXAMPLE['id'])

        sot.delete_ip_addresses(self.sess, ip_list=[{'ip': '192.168.0.2'}])

        self.sess.post.assert_called_once_with(
            'elb/ipgroups/%s/iplist/batch-delete' % EXAMPLE['id'],
            json={'ipgroup': {'ip_list': [{'ip': '192.168.0.2'}]}}
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base
from otcextensions.sdk.ctsv3.v3 import trace

EXAMPLE = {
    "time": 1472148708232,
    "user": {
        "name": "xxx",
        "domain": {
            "name": "xxx",
            "id": "ded649d814464428ba89d04d7955c93e"
        }
    },
    "response": {
        "code": "VPC.0514",
        "message": "Update port fail."
    },
    "code": 200,
    "service_type": "VPC",
    "resource_type": "eip",
    "resource_name": "192.144.163.1",
    "resource_id": "d502809d-0d1d-41ce-9690-784282142ccc",
    "trace_name": "deleteEip",
    "trace_rating": "warning",
    "trace_type": "ConsoleAction",
    "api_version": "2.0",
    "record_time": 1481066128032,
    "trace_id": "e001ccb9-bc09-11e6-b00b-4b2a61338db6"
}


class TestTrace(base.TestCase):
    def test_basic(self):
        sot = trace.Trace()
        self.assertEqual('/traces', sot.base_path)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):
        sot = trace.Trace(**EXAMPLE)
        self.assertEqual(EXAMPLE['time'], sot.time)
        self.assertEqual(EXAMPLE['user']['name'], sot.user.name)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class ApiEnvironment(resource.Resource):
    base_path = f'/apigw/instances/%(gateway_id)s/envs'

    allow_list = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    resources_key = 'envs'

    _query_mapping = resource.QueryParameters('limit', 'offset', 'name')

    gateway_id = resource.URI('gateway_id')
    name = resource.Body('name')
    remark = resource.Body('remark')

    create_time = resource.Body('create_time')
    id = resource.Body('id')

    def _update_env(self, session, gateway, **attrs):
        """Update environment.
        """
        url = f'/apigw/instances/{gateway.id}/envs/{self.id}'
        response = session.put(url, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import csv
import base64
import binascii
from pathlib import Path

from openstack import proxy
# from openstack import exceptions
from otcextensions.sdk.dis.v2 import app as _app
from otcextensions.sdk.dis.v2 import checkpoint as _checkpoint
from otcextensions.sdk.dis.v2 import data as _data
from otcextensions.sdk.dis.v2 import dump_task as _dump_task
from otcextensions.sdk.dis.v2 import stream as _stream


class Proxy(proxy.Proxy):

    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {"Accept": "application/json",
                                   "Content-type": "application/json"}

    # ======== Stream ========
    def streams(self, **params):
        """List all Streams.

        :param dict params: Optional query parameters to be sent to limit
            the streams being returned.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dis.v2.stream.Stream`) instances
        """
        if params.get('limit'):
            params.update(paginated=False)
        return self._list(_stream.Stream, **params)

    def get_stream(self, stream):
        """Query details of a DIS stream

        :param stream: The value can be the name of a DIS Stream.
        :returns: One :class:`~otcextensions.sdk.dis.v2.stream.Stream`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_stream.Stream, stream)

    def create_stream(self, **attrs):
        """Create a stream from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dis.v2.stream.Stream`,
            comprised of the properties on the Streams class.
        :returns: The results of stream creation
        :rtype: :class:`~otcextensions.sdk.dis.v2.stream.Stream`
        """
        return self._create(_stream.Stream, **attrs)

    def delete_stream(self, stream_name, ignore_missing=False):
        """Delete a DIS stream.

        :param stream_name: Name of a DIS stream.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the stream does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent stream.

        :returns: ``None``
        """
        return self._delete(_stream.Stream, stream_name,
                            ignore_missing=ignore_missing)

    def update_stream_partition(self, stream_name, count):
        """Update a DIS stream partition count.

        :param stream_name: Name of a DIS stream.
        :param count: Number of the target partitions.

        :returns: ``None``
        """
        attrs = {
            'stream_name': stream_name,
            'target_partition_count': count
        }

        return self._update(_stream.Stream, stream_name, **attrs)

    # ======== App ========
    def create_app(self, **attrs):
        """Create an App from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dis.v2.app.App`,
            comprised of the properties on the App class.
        :returns: The results of app creation
        :rtype: :class:`~otcextensions.sdk.dis.v2.app.App`
        """
        return self._create(_app.App, **attrs)

    def apps(self, **params):
        """List all Apps.

        :param dict params: Optional query parameters to be sent to limit
            the apps being returned.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dis.v2.app.App`) instances
        """
        if params.get('limit'):
            params.update(paginated=False)
        return self._list(_app.App, **params)

    def get_app(self, app_name):
        """Query details of a DIS App.

        :param app_name: Name of the app to be queried.
        :returns: One :class:`~otcextensions.sdk.dis.v2.app.App`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_app.App, app_name)

    def app_consumptions(self, stream_name, app_name, **params):
        """List partition consuming state list.

        :param stream_name: Name of the stream.
        :param app_name: Name of the app to be queried.
        :param dict params: Optional query parameters to be sent to limit
            the app consumptions being returned.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dis.v2.app.App`) instances
        """
        params.setdefault('checkpoint_type', 'LAST_READ')
        if params.get('limit'):
            params.update(paginated=False)
        return self._list(_app.AppConsumption, app_name=app_name,
                          stream_name=stream_name, **params)

    def delete_app(self, app_name, ignore_missing=False):
        """Delete a DIS App.

        :param app_name: The value can be the name of a DIS app.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the app does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent app.

        :returns: ``None``
        """
        return self._delete(_app.App, app_name, ignore_missing=ignore_missing)

    # ======== Stream ========
    def create_checkpoint(self, **attrs):
        """Add a Checkpoint from attributes.

        :param dict attrs: Keyword arguments which will be used to submit
            a :class:`~otcextensions.sdk.dis.v2.checkpoint.Checkpoint`,
            comprised of the properties on the Checkpoint class.
        :returns: The results of stream creation
        :rtype: :class:`~otcextensions.sdk.dis.v2.checkpoint.Checkpoint`
        """
        attrs.update(checkpoint_type='LAST_READ')
        return self._create(_checkpoint.Checkpoint, **attrs)

    def get_checkpoint(self, stream_name, app_name,
                       partition_id, checkpoint_type='LAST_READ'):
        """Query details of a Checkpoint.

        :param stream_name: Name of the stream to which the checkpoint belongs.
        :param app_name: Name of the app associated with the checkpoint.
        :param partition_id: Identifier of the stream partition.
        :param checkpoint_type: Type of the checkpoint (Default: LAST_READ).
        :returns: One :class:`~otcextensions.sdk.dis.v2.checkpoint.Checkpoint`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        params = {
            'stream_name': stream_name,
            'app_name': app_name,
            'partition_id': partition_id,
            'checkpoint_type': checkpoint_type
        }
        obj = self._get_resource(_checkpoint.Checkpoint, None)
        return obj.get_checkpoint(self, **params)

    def delete_checkpoint(self, stream_name, app_name,
                          partition_id=None, checkpoint_type='LAST_READ'):
        """Delete a Checkpoint.

        :param stream_name: Name of the stream to which the checkpoint belongs.
        :param app_name: Name of the app associated with the checkpoint.
        :param partition_id: Identifier of the stream partition (Default: None)
        :param checkpoint_type: Type of the checkpoint (Default: LAST_READ).

        :returns: ``None``
        """
        params = {
            'stream_name': stream_name,
            'app_name': app_name,
            'checkpoint_type': checkpoint_type
        }
        if partition_id:
            params.update(partition_id=partition_id)

        obj = self._get_resource(_checkpoint.Checkpoint, None)
        return obj.delete_checkpoint(self, **params)

    # ======== Data Management ========
    def upload_data(self, stream_name, stream_id=None,
                    records=None, filename=None):
        """Upload data to DIS stream.

        :param stream_name: Name of the stream.
        :param stream_id: Optional stream ID.
        :param records: List of records (if filename is not used).
        :param filename: Path to the CSV file.

        :returns: The results of uploaded data.
        :rtype: :class:`~otcextensions.sdk.dis.v2.data.Data`
        """
        def encode_data(data):
            try:
                base64.b64decode(data, validate=True)
                return data
            except binascii.Error:
                return base64.b64encode(data.encode('ascii')).decode('ascii')

        records = records if records else []
        records_data = []
        if filename:
            with Path(filename).open('r') as csv_content:
                header = next(csv.reader(csv_content, delimiter=','))
                if 'data' not in header:
                    raise ValueError(
                        f'data column is missing in the header of {filename}'
                    )
                reader = csv.DictReader(csv_content, header, delimiter=',')
                for record in reader:
                    data = record.get('data')
                    if data and data != '':
                        record['data'] = encode_data(data)
                    else:
                        raise ValueError('Data File contains empty data!')
                    records_data.append(record)
        else:
            for record in records:
                data = record.get('data')
                if data and data != '':
                    record['data'] = encode_data(data)
                else:
                    raise ValueError('data is missing in the attributes')
                records_data.append(record)

        request_attrs = {
            'stream_name': stream_name,
            'records': records_data
        }
        if stream_id:
            request_attrs.append(stream_id=stream_id)
        return self._create(_data.Data, **request_attrs)

    def download_data(self, partititon_cursor,
                      max_fetch_bytes=None, filename=None):
        """Download data from a DIS stream.

        :param partititon_cursor: Data cursor, which needs to be
            obtained through the API for obtaining data cursors.
        :param max_fetch_bytes: Maximum number of bytes
            that can be obtained for each request.
        :param filename: Path to the CSV file.

        :returns: a generator of
            (:class:`~otcextensions.sdk.dis.v2.data.Data`) instances
        """
        params = {'partition-cursor': partititon_cursor}
        if max_fetch_bytes:
            params.update(max_fetch_bytes=max_fetch_bytes)
        data = self._list(_data.Data, **params)
        if filename:
            columns = (
                'sequence_number',
                'data',
                'timestamp',
                'timestamp_type',
            )
            with open(filename, 'w', encoding='UTF8', newline='') as f:
                writer = csv.writer(f)
                # write the header
                writer.writerow(columns)
                # write multiple rows
                writer.writerows(
                    (s.sequence_number, s.data, s.timestamp,
                        s.timestamp_type) for s in data
                )
        else:
            return data

    def get_data_cursor(self, stream_name, partition_id, **params):
        """Query data cursor.

        :param stream_name: Name of the stream.
        :param partititon_id: Partition ID of the stream.
        :param dict params: Optional query parameters.

        :returns: `partition_cursor`.
        :rtype: :class:`~otcextensions.sdk.dis.v2.data.Data`
        """
        params['stream-name'] = stream_name
        params['partition-id'] = partition_id
        obj = _data.Data('')
        return obj.get_data_cursor(self, **params)

    # ======== Stream ========
    def create_dump_task(self, stream_name, **attrs):
        """Add OBS dump tasks.

        :param stream_name: Name of the stream.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.dis.v2.dump_task.DumpTask`,
            comprised of the properties on the DumpTask class.
        :returns: The results of adding DumpTask.
        :rtype: :class:`~otcextensions.sdk.dis.v2.dump_task.DumpTask`
        """
        return self._create(
            _dump_task.DumpTask, uri_stream_name=stream_name, **attrs
        )

    def dump_tasks(self, stream_name):
        """List Dump tasks.

        :param stream_name: Name of the stream.
        :returns: a generator of
            (:class:`~otcextensions.sdk.dis.v2.dump_task.DumpTask`) instances
        """
        return self._list(_dump_task.DumpTask, uri_stream_name=stream_name)

    def delete_dump_task(self, stream_name, task_name, ignore_missing=False):
        """Delete Dump Task.

        :param stream_name: Name of the stream.
        :param task_name: Name of the dump task to be deleted.

        :returns: ``None``
        """
        return self._delete(_dump_task.DumpTask, task_name,
                            uri_stream_name=stream_name,
                            ignore_missing=ignore_missing)

    def get_dump_task(self, stream_name, task_name):
        """Query Dump Task details.

        :param stream_name: Name of the stream..
        :param task_name: Name of the dump task to be deleted.
        :returns: One :class:`~otcextensions.sdk.dis.v2.dump_task.DumpTask`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.
        """
        return self._get(_dump_task.DumpTask, task_name,
                         uri_stream_name=stream_name)

    def start_dump_task(self, stream_name, *task_id):
        """Start dump tasks in batches.

        :param stream_name: Name of the stream.
        :param task_id: Dump task ID(s).
        :returns: ``None``
        """
        obj = _dump_task.DumpTask()
        return obj._action(self, stream_name, 'start', *task_id)

    def pause_dump_task(self, stream_name, *task_id):
        """Pause dump tasks in batches.

        :param stream_name: Name of the stream.
        :param task_id: Dump task ID(s).
        :returns: ``None``
        """
        obj = _dump_task.DumpTask()
        return obj._action(self, stream_name, 'stop', *task_id)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

from otcextensions.osclient.cbr.v3 import checkpoint
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestCheckpoint(fakes.TestCBR):

    def setUp(self):
        super(TestCheckpoint, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeCheckpoint.create_one()

        flat_data = checkpoint._flatten_checkpoint(obj)

        data = (
            flat_data['created_at'],
            flat_data['id'],
            flat_data['status'],
            flat_data['vault_id'],
            flat_data['vault_name'],
            flat_data['backup_name']
        )

        vault = obj.vault
        ei = obj.extra_info
        cmp_data = (
            obj.created_at,
            obj.id,
            obj.status,
            vault.id,
            vault.name,
            ei.name
        )

        self.assertEqual(data, cmp_data)

    def test_normalize_resource_details(self):
        resource_details = [
            'id=7238-2378 type=OS::Nova::Server name=name-1',
            'id=9238-2378 type=OS::Nova::Server name=name-2'
        ]

        verify_result = [
            {'id': '7238-2378', 'name': 'name-1', 'type': 'OS::Nova::Server'},
            {'id': '9238-2378', 'name': 'name-2', 'type': 'OS::Nova::Server'}
        ]

        result = checkpoint._normalize_resources(resource_details)

        self.assertEqual(result, verify_result)


class TestShowCheckpoint(fakes.TestCBR):

    object = fakes.FakeCheckpoint.create_one()

    columns = (
        'id',
        'status',
        'created_at',
        'vault_id',
        'vault_name',
        'backup_name'
    )

    flat_data = checkpoint._flatten_checkpoint(object)

    data = (
        flat_data['id'],
        flat_data['status'],
        flat_data['created_at'],
        flat_data['vault_id'],
        flat_data['vault_name'],
        flat_data['backup_name']
    )

    def setUp(self):
        super(TestShowCheckpoint, self).setUp()

        self.cmd = checkpoint.ShowCheckpoint(self.app, None)

        self.client.get_checkpoint = mock.Mock()

    def test_default(self):
        arglist = [
            'checkpoint'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.get_checkpoint.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.get_checkpoint.assert_called_once_with(
            checkpoint='checkpoint'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestCreateCheckpoint(fakes.TestCBR):

    object = fakes.FakeCheckpoint.create_one()

    columns = (
        'id',
        'status',
        'created_at',
        'vault_id',
        'vault_name',
        'backup_name',
    )

    flat_data = checkpoint._flatten_checkpoint(object)

    data = (
        flat_data['id'],
        flat_data['status'],
        flat_data['created_at'],
        flat_data['vault_id'],
        flat_data['vault_name'],
        flat_data['backup_name']
    )

    def setUp(self):
        super(TestCreateCheckpoint, self).setUp()

        self.cmd = checkpoint.CreateCheckpoint(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.create_checkpoint = mock.Mock()

    def test_default(self):
        arglist = [
            '--vault-id', 'vault_id',
            '--auto-trigger',
            '--description', 'description',
            '--no-incremental',
            '--backup-name', 'backup_name',
            '--resources', 'resource-1-uuid',
            '--resources', 'resource-2-uuid',
            '--resource-details', 'id=uuid'
                                  ' name=name type=OS::Nova::Server'
        ]
        verifylist = [
            ('vault_id', 'vault_id'),
            ('auto_trigger', True),
            ('description', 'description'),
            ('no_incremental', False),
            ('backup_name', 'backup_name'),
            ('resources', ['resource-1-uuid', 'resource-2-uuid']),
            ('resource_details', ['id=uuid name=name type=OS::Nova::Server'])
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_checkpoint.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_checkpoint.assert_called_once_with(
            parameters={
                'auto_trigger': True,
                'description': 'description',
                'incremental': False,
                'name': 'backup_name',
                'resources': ['resource-1-uuid', 'resource-2-uuid'],
                'resource_details': [{'id': 'uuid', 'name': 'name',
                                      'type': 'OS::Nova::Server'}]},
            vault_id='vault_id',
        )

        self.data, self.columns = checkpoint._add_resources_to_obj(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CBR Task CLI implementation'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_task(obj):
    """Flatten the structure of the task into a single dict
    """

    data = {
        'id': obj.id,
        'checkpoint_id': obj.checkpoint_id,
        'policy_id': obj.policy_id,
        'provider_id': obj.provider_id,
        'vault_id': obj.vault_id,
        'vault_name': obj.vault_name,
        'operation_type': obj.operation_type,
        'error_mesage': obj.error_info.message,
        'error_code': obj.error_info.code,
        'created_at': obj.created_at,
        'ended_at': obj.ended_at,
        'started_at': obj.started_at,
        'updated_at': obj.updated_at,
    }

    return data


class ListTasks(command.Lister):
    _description = _('List CBR Tasks')
    columns = ('id', 'checkpoint_id', 'provider_id',
               'operation_type', 'created_at', 'ended_at')

    def get_parser(self, prog_name):
        parser = super(ListTasks, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        data = client.tasks()

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_task(s), self.columns,
                 ) for s in data))
        return table


class ShowTask(command.ShowOne):
    _description = _('Show single Task details')
    columns = (
        'id',
        'checkpoint_id',
        'policy_id',
        'provider_id',
        'vault_id',
        'vault_name',
        'operation_type',
        'error_mesage',
        'error_code',
        'created_at',
        'ended_at',
        'started_at',
        'updated_at'
    )

    def get_parser(self, prog_name):
        parser = super(ShowTask, self).get_parser(prog_name)
        parser.add_argument(
            'operation_log_id',
            metavar='<operation_log>',
            help=_('ID of the CBR task.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cbr

        obj = client.get_task(
            parsed_args.operation_log_id,
        )

        data = utils.get_dict_properties(
            _flatten_task(obj), self.columns)

        return self.columns, data



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
"""
Create a DNAT Rule
"""
import openstack


openstack.enable_logging(True)
conn = openstack.connect(cloud='otc')

nat_gateway_id = 'nat_gateway_id'
port_id = 'network_id'
private_ip = '192.168.199.3'
floating_ip_id = 'floating_ip_id'
protocol = 'TCP'
internal_service_port = 80
external_service_port = 80


attrs = {
    "nat_gateway_id": nat_gateway_id,
    "private_ip": private_ip,
    "port_id": port_id,
    "protocol": protocol,
    "internal_service_port": internal_service_port,
    "external_service_port": external_service_port,
    "floating_ip_id": floating_ip_id
}

dnat_rule = conn.nat.create_dnat_rule(**attrs)
print(dnat_rule)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from otcextensions.sdk.apig.v2 import _proxy
from otcextensions.sdk.apig.v2 import gateway as _gateway
from otcextensions.sdk.apig.v2 import az as _az
from otcextensions.sdk.apig.v2 import apienvironment as _env
from otcextensions.sdk.apig.v2 import apienvironmentvar as _var
from otcextensions.sdk.apig.v2 import apigroup as _api_group
from otcextensions.sdk.apig.v2 import throttling_policy as _tp
from otcextensions.sdk.apig.v2 import api
from otcextensions.sdk.apig.v2 import api_supplements as _as
from otcextensions.sdk.apig.v2 import signature as _sign
from otcextensions.sdk.apig.v2 import signature_binding as _sb
from otcextensions.sdk.apig.v2 import throttling_policy_binding as _tb
from otcextensions.sdk.apig.v2 import throttling_excluded as _tx
from otcextensions.sdk.apig.v2 import gateway_features as _gwf
from otcextensions.sdk.apig.v2 import domain_name
from otcextensions.sdk.apig.v2 import certificate
from otcextensions.sdk.apig.v2 import resource_query as _rq
from otcextensions.sdk.apig.v2 import app as _app
from otcextensions.sdk.apig.v2 import appcode as _appcode
from otcextensions.sdk.apig.v2 import api_auth as _auth
from otcextensions.sdk.apig.v2 import acl_policy as _ac
from otcextensions.sdk.apig.v2 import acl_api_binding as _acl_api
from otcextensions.sdk.apig.v2 import custom_authorizer as _custom_auth
from otcextensions.sdk.apig.v2 import vpc_channel as _vpc
from otcextensions.sdk.apig.v2 import backend_server_group as _vpc_sg
from otcextensions.sdk.apig.v2 import backend_server as _vpc_s
from openstack.tests.unit import test_proxy_base
from unittest import mock


class TestApiGatewayProxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestApiGatewayProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestApiGatewayFunctions(TestApiGatewayProxy):
    def test_gateways(self):
        self.verify_list(self.proxy.gateways, _gateway.Gateway)

    def test_azs(self):
        self.verify_list(self.proxy.azs, _az.AZ)

    def test_create_gateway(self):
        self.verify_create(self.proxy.create_gateway,
                           _gateway.Gateway)

    def test_delete_gateway(self):
        gateway = _gateway.Gateway()
        self.verify_delete(self.proxy.delete_gateway,
                           _gateway.Gateway,
                           method_args=[gateway],
                           expected_args=[gateway]
                           )

    def test_get_gateway(self):
        self.verify_get(self.proxy.get_gateway,
                        _gateway.Gateway)

    def test_update_gateway(self):
        self.verify_update(self.proxy.update_gateway,
                           _gateway.Gateway)

    def test_get_gateway_progress(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._get_creation_progress',
            self.proxy.get_gateway_progress,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_modify_spec(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._modify_spec',
            self.proxy.modify_gateway_spec,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_get_constraints(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._get_constraints',
            self.proxy.get_constraints,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_enable_public_access(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._enable_public_access',
            self.proxy.enable_public_access,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_update_public_access(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._update_public_access',
            self.proxy.update_public_access,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_disable_public_access(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._disable_public_access',
            self.proxy.disable_public_access,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_bind_eip(self):
        gateway = _gateway.Gateway()
        self.proxy._get_resource = mock.Mock(return_value=gateway)
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._bind_eip',
            self.proxy.bind_eip,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_unbind_eip(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._unbind_eip',
            self.proxy.unbind_eip,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_enable_ingress(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._enable_ingress',
            self.proxy.enable_ingress,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_update_ingress(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._update_ingress',
            self.proxy.update_ingress,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_disable_ingress(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway.'
            'Gateway._disable_ingress',
            self.proxy.disable_ingress,
            method_args=[gateway],
            expected_args=[self.proxy, gateway]
        )

    def test_create_env(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_environment,
                           _env.ApiEnvironment,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_update_env(self):
        gateway = _gateway.Gateway()
        env = _env.ApiEnvironment()
        self._verify(
            'otcextensions.sdk.apig.v2.apienvironment.'
            'ApiEnvironment._update_env',
            self.proxy.update_environment,
            method_args=[gateway, env],
            expected_args=[self.proxy, gateway]
        )

    def test_delete_env(self):
        gateway = _gateway.Gateway()
        env = _env.ApiEnvironment()
        self.verify_delete(self.proxy.delete_environment,
                           _env.ApiEnvironment,
                           method_args=[gateway, env],
                           expected_args=[gateway],
                           expected_kwargs={'gateway_id': None}
                           )

    def test_list_envs(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.environments,
                         _env.ApiEnvironment,
                         method_args=[gateway],
                         expected_args=[],
                         expected_kwargs={'gateway_id': None}
                         )

    def test_create_api_group(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_api_group,
                           _api_group.ApiGroup,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_update_api_group(self):
        gateway = _gateway.Gateway()
        api_group = _api_group.ApiGroup()
        self._verify(
            'otcextensions.sdk.apig.v2.apigroup.'
            'ApiGroup._update_group',
            self.proxy.update_api_group,
            method_args=[gateway, api_group],
            expected_args=[self.proxy],
            expected_kwargs={'gateway': gateway}
        )

    def test_delete_api_group(self):
        gateway = _gateway.Gateway()
        api_group = _api_group.ApiGroup()
        self.verify_delete(self.proxy.delete_api_group,
                           _api_group.ApiGroup,
                           method_args=[gateway, api_group],
                           expected_args=[gateway],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_list_api_groups(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.api_groups,
                         _api_group.ApiGroup,
                         method_args=[gateway],
                         expected_args=[],
                         expected_kwargs={'gateway_id': None}
                         )

    def test_get_api_group(self):
        gateway = _gateway.Gateway()
        api_group = _api_group.ApiGroup()
        self.verify_get(self.proxy.get_api_group,
                        _api_group.ApiGroup,
                        method_args=[gateway, api_group],
                        expected_args=[api_group],
                        expected_kwargs={'gateway_id': None}
                        )

    def test_verify_name(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.apigroup.'
            'ApiGroup._verify_name',
            self.proxy.verify_api_group_name,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway': gateway}
        )


class TestApiEnvVars(TestApiGatewayProxy):
    def test_environment_variables(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.environment_variables,
            _var.ApiEnvironmentVar,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_create_environment_variable(self):
        gateway = _gateway.Gateway()
        attrs = {
            "variable_name": "address",
            "variable_value": "192.168.1.5",
            "env_id": "env_id",
            "group_id": "group_id"
        }
        self.verify_create(
            self.proxy.create_environment_variable,
            _var.ApiEnvironmentVar,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_delete_environment_variable(self):
        gateway = _gateway.Gateway()
        var = _var.ApiEnvironmentVar()
        self.verify_delete(
            self.proxy.delete_environment_variable,
            _var.ApiEnvironmentVar,
            method_args=[gateway, var],
            expected_args=[var],
            expected_kwargs={"gateway_id": None}
        )

    def test_get_environment_variable(self):
        gateway = _gateway.Gateway()
        var = _var.ApiEnvironmentVar()
        self.verify_get(
            self.proxy.get_environment_variable,
            _var.ApiEnvironmentVar,
            method_args=[gateway, var],
            expected_args=[var],
            expected_kwargs={"gateway_id": None}
        )

    def test_update_environment_variable(self):
        gateway = _gateway.Gateway()
        var = _var.ApiEnvironmentVar()
        attrs = {
            "variable_value": "192.168.1.5",
        }
        self.verify_update(
            self.proxy.update_environment_variable,
            _var.ApiEnvironmentVar,
            method_args=[gateway, var],
            expected_args=[var],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )


class TestThrottlingPolicy(TestApiGatewayProxy):
    def test_throttling_policies(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.throttling_policies,
            _tp.ThrottlingPolicy,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_create_throttling_policy(self):
        gateway = _gateway.Gateway()
        attrs = {
            "api_call_limits": 100,
            "app_call_limits": 60,
            "enable_adaptive_control": "FALSE",
            "ip_call_limits": 60,
            "name": "throttle_demo",
            "remark": "Total: 800 calls/second;"
                      " user: 500 calls/second;"
                      " app: 300 calls/second;"
                      " IP address: 600 calls/second",
            "time_interval": 1,
            "time_unit": "SECOND",
            "type": 1,
            "user_call_limits": 60
        }
        self.verify_create(
            self.proxy.create_throttling_policy,
            _tp.ThrottlingPolicy,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_delete_throttling_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        self.verify_delete(
            self.proxy.delete_throttling_policy,
            _tp.ThrottlingPolicy,
            method_args=[gateway, policy],
            expected_args=[policy],
            expected_kwargs={"gateway_id": None}
        )

    def test_get_throttling_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        self.verify_get(
            self.proxy.get_throttling_policy,
            _tp.ThrottlingPolicy,
            method_args=[gateway, policy],
            expected_args=[policy],
            expected_kwargs={"gateway_id": None}
        )

    def test_update_throttling_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        attrs = {
            "time_unit": "SECOND",
            "name": "throttle_demo",
            "api_call_limits": 100,
            "time_interval": 1,
            "remark": "Total: 800 calls/second;"
                      " user: 500 calls/second;"
                      " app: 300 calls/second;"
                      " IP address: 600 calls/second",
        }
        self.verify_update(
            self.proxy.update_throttling_policy,
            _tp.ThrottlingPolicy,
            method_args=[gateway, policy],
            expected_args=[policy],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )


class TestApi(TestApiGatewayProxy):
    def test_apis(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.apis,
            api.Api,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_create_api(self):
        gateway = _gateway.Gateway()
        attrs = {
            "group_id": "id",
            "name": "test_api_001",
            "auth_type": "IAM",
            "backend_type": "HTTP",
            "req_protocol": "HTTP",
            "req_uri": "/test/http",
            "remark": "Mock backend API",
            "type": 2,
            "req_method": "GET",
            "result_normal_sample": "Example success response",
            "result_failure_sample": "Example failure response",
            "tags": ["httpApi"],
            "backend_api": {
                "req_protocol": "HTTP",
                "req_method": "GET",
                "req_uri": "/test/benchmark",
                "timeout": 5000,
                "retry_count": "-1",
                "url_domain": "192.168.189.156:12346"
            },
        }
        self.verify_create(
            self.proxy.create_api,
            api.Api,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_delete_api(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        self.verify_delete(
            self.proxy.delete_api,
            api.Api,
            method_args=[gateway, a],
            expected_args=[a],
            expected_kwargs={"gateway_id": None}
        )

    def test_get_api(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        self.verify_get(
            self.proxy.get_api,
            api.Api,
            method_args=[gateway, a],
            expected_args=[a],
            expected_kwargs={"gateway_id": None}
        )

    def test_update_api(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        attrs = {
            "group_id": "id",
            "name": "test_api_001",
            "auth_type": "IAM",
            "backend_type": "HTTP",
            "req_protocol": "HTTP",
            "req_uri": "/test/http",
            "remark": "Mock backend API",
            "type": 2,
            "req_method": "GET",
            "result_normal_sample": "Example success response",
            "result_failure_sample": "Example failure response",
            "tags": ["httpApi"],
            "backend_api": {
                "req_protocol": "HTTP",
                "req_method": "GET",
                "req_uri": "/test/benchmark",
                "timeout": 5000,
                "retry_count": "-1",
                "url_domain": "192.168.189.156:12346"
            },
        }
        self.verify_update(
            self.proxy.update_api,
            api.Api,
            method_args=[gateway, a],
            expected_args=[a],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )


class TestApiSupplements(TestApiGatewayProxy):
    def test_online_api(self):
        gateway = _gateway.Gateway()
        environment = _env.ApiEnvironment()
        a = api.Api()
        self._verify(
            'otcextensions.sdk.apig.v2.api_supplements.'
            'PublishApi.publish_api',
            self.proxy.publish_api,
            method_args=[gateway, environment, a],
            expected_args=[self.proxy],
            expected_kwargs={
                "api_id": None,
                "env_id": None,
                "gateway_id": None
            }
        )

    def test_offline_api(self):
        gateway = _gateway.Gateway()
        environment = _env.ApiEnvironment()
        a = api.Api()
        self._verify(
            'otcextensions.sdk.apig.v2.api_supplements.'
            'PublishApi.take_api_offline',
            self.proxy.offline_api,
            method_args=[gateway, environment, a],
            expected_args=[self.proxy],
            expected_kwargs={
                "api_id": None,
                "env_id": None,
                "gateway_id": None
            }
        )

    def test_check_api(self):
        gateway = _gateway.Gateway()
        attrs = {
            "type": "name",
            "name": "test"
        }
        self.verify_create(
            self.proxy.check_api,
            _as.CheckApi,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None
            }
        )

    def test_debug_api(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        attrs = {
            "mode": "DEVELOPER",
            "scheme": "HTTP",
            "method": "GET",
            "path": "/test/http"
        }
        self.verify_create(
            self.proxy.debug_api,
            _as.DebugApi,
            method_args=[gateway, a],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "api_id": None,
                "gateway_id": None
            }
        )

    def test_online_apis(self):
        gateway = _gateway.Gateway()
        environment = _env.ApiEnvironment()
        self._verify(
            'otcextensions.sdk.apig.v2.api_supplements.'
            'PublishApis.publish_apis',
            self.proxy.publish_apis,
            method_args=[gateway, environment],
            expected_args=[self.proxy],
            expected_kwargs={
                "env_id": None,
                "gateway_id": None
            }
        )

    def test_offline_apis(self):
        gateway = _gateway.Gateway()
        environment = _env.ApiEnvironment()
        self._verify(
            'otcextensions.sdk.apig.v2.api_supplements.'
            'PublishApis.take_apis_offline',
            self.proxy.offline_apis,
            method_args=[gateway, environment],
            expected_args=[self.proxy],
            expected_kwargs={
                "env_id": None,
                "gateway_id": None
            }
        )

    def test_api_versions(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        self.verify_list(
            self.proxy.api_versions,
            _as.PublishApis,
            method_args=[gateway, a],
            expected_args=[],
            expected_kwargs={
                "api_id": a,
                "gateway_id": None
            }
        )

    def test_switch_version(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        version_id = 'id'
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.switch_version,
            # expected_result=_as.PublishApis(),
            method_args=[gateway, a, version_id],
            method_kwargs={},
            expected_args=[_as.PublishApis],
            expected_kwargs={
                "id": a,
                "version_id": 'id',
                "gateway_id": None
            }
        )

    def test_api_runtime_definitions(self):
        gateway = _gateway.Gateway()
        a = api.Api()
        self.verify_list(
            self.proxy.api_runtime_definitions,
            _as.RuntimeDefinitionApi,
            method_args=[gateway, a],
            expected_args=[],
            expected_kwargs={
                "api_id": a,
                "gateway_id": None
            }
        )

    def test_api_version_details(self):
        gateway = _gateway.Gateway()
        version_id = 'id'
        self.verify_list(
            self.proxy.api_version_details,
            _as.VersionsApi,
            method_args=[gateway, version_id],
            expected_args=[],
            expected_kwargs={
                "version_id": 'id',
                "gateway_id": None
            }
        )

    def test_take_api_version_offline(self):
        gateway = _gateway.Gateway()
        version_id = 'id'
        self.verify_delete(
            self.proxy.take_api_version_offline,
            _as.VersionsApi,
            method_args=[gateway, version_id],
            expected_args=[],
            expected_kwargs={
                "version_id": 'id',
                "gateway_id": None
            }
        )


class TestSignature(TestApiGatewayProxy):
    def test_signatures(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.signatures,
            _sign.Signature,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_create_signature(self):
        gateway = _gateway.Gateway()
        attrs = {
            "name": "otce_signature_1",
            "sign_type": "aes",
            "sign_algorithm": "aes-256-cfb",
        }
        self.verify_create(
            self.proxy.create_signature,
            _sign.Signature,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_delete_signature(self):
        gateway = _gateway.Gateway()
        s = _sign.Signature()
        self.verify_delete(
            self.proxy.delete_signature,
            _sign.Signature,
            method_args=[gateway, s],
            expected_args=[s],
            expected_kwargs={"gateway_id": None}
        )

    def test_update_api(self):
        gateway = _gateway.Gateway()
        s = _sign.Signature()
        attrs = {
            "name": "otce_signature_1",
            "sign_type": "aes",
            "sign_algorithm": "aes-128-cfb",
        }
        self.verify_update(
            self.proxy.update_signature,
            _sign.Signature,
            method_args=[gateway, s],
            expected_args=[s],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )


class TestSignatureBind(TestApiGatewayProxy):
    def test_bound_signatures(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.bound_signatures,
            _sb.SignatureBind,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_not_bound_apis(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.not_bound_apis,
            _sb.NotBoundApi,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_bound_apis(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.bound_apis,
            _sb.BoundApi,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_bind_signature(self):
        gateway = _gateway.Gateway()
        attrs = {
            "name": "otce_signature",
            "sign_type": "aes",
            "sign_algorithm": "aes-256-cfb",
        }
        self.verify_create(
            self.proxy.bind_signature,
            _sb.SignatureBind,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_unbind_signature(self):
        gateway = _gateway.Gateway()
        s = _sb.SignatureBind()
        self.verify_delete(
            self.proxy.unbind_signature,
            _sb.SignatureBind,
            method_args=[gateway, s],
            expected_args=[s],
            expected_kwargs={"gateway_id": None}
        )


class TestThrottlesBind(TestApiGatewayProxy):
    def test_bound_throttling_policies(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.bound_throttling_policies,
            _tb.BoundThrottles,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_not_bound_throttling_policy_apis(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.not_bound_throttling_policy_apis,
            _tb.NotBoundApi,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_bound_throttling_policy_apis(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.bound_throttling_policy_apis,
            _tb.ThrottlingPolicyBind,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_bind_throttling_policy(self):
        gateway = _gateway.Gateway()
        attrs = {
            "throttle_id": "id",
            "publish_ids": ["publish_id"]
        }
        self.verify_create(
            self.proxy.bind_throttling_policy,
            _tb.ThrottlingPolicyBind,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_unbind_throttling_policy(self):
        gateway = _gateway.Gateway()
        t = _tb.ThrottlingPolicyBind()
        self.verify_delete(
            self.proxy.unbind_throttling_policy,
            _tb.ThrottlingPolicyBind,
            method_args=[gateway, t],
            expected_args=[t],
            expected_kwargs={"gateway_id": None}
        )

    def test_unbind_throttling_policies(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.throttling_policy_binding.'
            'ThrottlingPolicyBind.unbind_policies',
            self.proxy.unbind_throttling_policies,
            method_args=[gateway, ["t"]],
            expected_args=[self.proxy],
            expected_kwargs={"gateway_id": None, "throttle_bindings": ["t"]}
        )


class TestThrottlingExcludePolicy(TestApiGatewayProxy):
    def test_throttling_excluded_policies(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        self.verify_list(
            self.proxy.throttling_excluded_policies,
            _tx.ThrottlingExcludedPolicy,
            method_args=[gateway, policy],
            expected_args=[],
            expected_kwargs={
                "gateway_id": None,
                "throttle_id": None
            }
        )

    def test_create_throttling_excluded_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        attrs = {
            "call_limits": 50,
            "object_id": "id",
            "object_type": "USER"
        }
        self.verify_create(
            self.proxy.create_throttling_excluded_policy,
            _tx.ThrottlingExcludedPolicy,
            method_args=[gateway, policy],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None,
                "throttle_id": None
            }
        )

    def test_delete_throttling_excluded_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        ex = _tx.ThrottlingExcludedPolicy()
        self.verify_delete(
            self.proxy.delete_throttling_excluded_policy,
            _tx.ThrottlingExcludedPolicy,
            method_args=[gateway, policy, ex],
            expected_args=[ex],
            expected_kwargs={
                "gateway_id": None,
                "throttle_id": None
            }
        )

    def test_update_throttling_excluded_policy(self):
        gateway = _gateway.Gateway()
        policy = _tp.ThrottlingPolicy()
        ex = _tx.ThrottlingExcludedPolicy()
        attrs = {
            "call_limits": 30
        }
        self.verify_update(
            self.proxy.update_throttling_excluded_policy,
            _tx.ThrottlingExcludedPolicy,
            method_args=[gateway, policy, ex],
            expected_args=[ex],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None,
                "throttle_id": None
            }
        )


class TestGwFeatures(TestApiGatewayProxy):
    def test_list_gateway_features(self):
        gateway = _gateway.Gateway()
        self.verify_list(
            self.proxy.gateway_features,
            _gwf.GatewayFeatures,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={"gateway_id": None}
        )

    def test_configure_gateway_feature(self):
        gateway = _gateway.Gateway()
        attrs = {
            "name": "route",
            "enable": False,
            "config": "{\"user_routes\":[]}",
        }
        self.verify_create(
            self.proxy.configure_gateway_feature,
            _gwf.GatewayFeatures,
            method_args=[gateway],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={**attrs, "gateway_id": None}
        )

    def test_supported_gateway_features(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.gateway_features.'
            'GatewayFeatures._supported_features',
            self.proxy.supported_gateway_features,
            method_args=[gateway],
            expected_args=[self.proxy, gateway],
            expected_kwargs={}
        )


class TestResourceQuery(TestApiGatewayProxy):
    def test_get_api_quantities(self):
        gateway = _gateway.Gateway()
        self.verify_get(
            self.proxy.get_api_quantities,
            _rq.ApiQuantities,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={
                'gateway_id': None,
                'requires_id': False
            }
        )

    def test_get_api_group_quantities(self):
        gateway = _gateway.Gateway()
        self.verify_get(
            self.proxy.get_api_group_quantities,
            _rq.ApiGroupQuantities,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={
                'gateway_id': None,
                'requires_id': False
            }
        )

    def test_get_app_quantities(self):
        gateway = _gateway.Gateway()
        self.verify_get(
            self.proxy.get_app_quantities,
            _rq.AppQuantities,
            method_args=[gateway],
            expected_args=[],
            expected_kwargs={
                'gateway_id': None,
                'requires_id': False
            }
        )


class TestDomain(TestApiGatewayProxy):
    def test_bind_domain_name(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        attrs = {
            "url_domain": "name"
        }
        self.verify_create(
            self.proxy.bind_domain_name,
            domain_name.DomainName,
            method_args=[gateway, group],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None,
                "group_id": None
            }
        )

    def test_unbind_domain_name(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        self.verify_delete(
            self.proxy.unbind_domain_name,
            domain_name.DomainName,
            method_args=[gateway, group, domain],
            expected_args=[domain],
            expected_kwargs={
                "gateway_id": None,
                "group_id": None,
                "ignore_missing": True
            }
        )

    def test_update_domain_name_bound(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        attrs = {
            "min_ssl_version": "TLSv1.2"
        }
        self.verify_update(
            self.proxy.update_domain_name_bound,
            domain_name.DomainName,
            method_args=[gateway, group, domain],
            expected_args=[domain],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None,
                "group_id": None
            }
        )

    def test_create_certificate_for_domain_name(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        attrs = {
            "name": "test",
            "private_key": "private_key",
            "cert_content": "content"
        }
        self.verify_create(
            self.proxy.create_certificate_for_domain_name,
            domain_name.Certificate,
            method_args=[gateway, group, domain],
            expected_args=[],
            method_kwargs={**attrs},
            expected_kwargs={
                **attrs,
                "gateway_id": None,
                "group_id": None,
                "domain_id": None
            }
        )

    def test_unbind_certificate_from_domain_name(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        cert = certificate.Certificate()
        self.verify_delete(
            self.proxy.unbind_certificate_from_domain_name,
            domain_name.DomainName,
            method_args=[gateway, group, domain, cert],
            expected_args=[domain],
            expected_kwargs={
                "gateway_id": None,
                "group_id": None,
                "domain_id": None,
                "certificate_id": None,
                "ignore_missing": True
            }
        )

    def test_enable_debug_domain_name(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        self._verify(
            'openstack.proxy.Proxy._update',
            self.proxy.enable_debug_domain_name,
            method_args=[gateway, group, domain, False],
            expected_args=[domain_name.DomainDebug],
            method_kwargs={},
            expected_kwargs={
                "gateway_id": None,
                "group_id": None,
                "domain_id": None,
                "sl_domain_access_enabled": False
            }
        )

    def test_get_bound_certificate(self):
        gateway = _gateway.Gateway()
        group = _api_group.ApiGroup()
        domain = domain_name.DomainName()
        cert = certificate.Certificate()
        self.verify_get(
            self.proxy.get_bound_certificate,
            domain_name.Certificate,
            method_args=[gateway, group, domain, cert],
            expected_args=[],
            method_kwargs={},
            expected_kwargs={
                "gateway_id": None,
                "group_id": None,
                "domain_id": None,
                "id": None
            }
        )


class TestCertificate(TestApiGatewayProxy):
    def test_delete_certificate(self):
        cert = certificate.Certificate()
        self.verify_delete(
            self.proxy.delete_certificate,
            certificate.Certificate,
            method_args=[cert],
            expected_args=[cert],
            expected_kwargs={
                "ignore_missing": True
            }
        )


class TestApp(TestApiGatewayProxy):
    def test_create_app(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_app,
                           _app.App,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_get_app(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_get(self.proxy.get_app,
                        _app.App,
                        method_args=[gateway, app],
                        expected_args=[app],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None}
                        )

    def test_update_app(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_update(self.proxy.update_app,
                           _app.App,
                           method_args=[gateway, app],
                           expected_args=[app],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_delete_app(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_delete(self.proxy.delete_app,
                           _app.App,
                           method_args=[gateway, app],
                           expected_args=[app],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_list_apps(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.apps,
                         _app.App,
                         method_args=[gateway, ],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_verify_app(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self._verify(
            'otcextensions.sdk.apig.v2.app.'
            'App._verify_app',
            self.proxy.verify_app,
            method_args=[gateway, app],
            expected_args=[self.proxy, gateway]
        )

    def test_reset_app_secret(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self._verify(
            'otcextensions.sdk.apig.v2.app.'
            'App._reset_secret',
            self.proxy.reset_app_secret,
            method_args=[gateway, app],
            expected_args=[self.proxy, gateway]
        )


class TestAppCode(TestApiGatewayProxy):
    def test_get_app_code(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        app_code = _appcode.AppCode()
        self.verify_get(self.proxy.get_app_code,
                        _appcode.AppCode,
                        method_args=[gateway, app, app_code],
                        expected_args=[app_code],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None,
                                         'app_id': None}
                        )

    def test_create_app_code(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_create(self.proxy.create_app_code,
                           _appcode.AppCode,
                           method_args=[gateway, app],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None,
                                            'app_id': None}
                           )

    def test_generate_app_code(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self._verify(
            'otcextensions.sdk.apig.v2.appcode.'
            'AppCode._generate_app_code',
            self.proxy.generate_app_code,
            method_args=[gateway, app],
            expected_args=[self.proxy, gateway, app]
        )

    def test_list_app_codes(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_list(self.proxy.app_codes,
                         _appcode.AppCode,
                         method_args=[gateway, app],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None,
                                          'app_id': None}
                         )

    def test_delete_app_code(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_delete(self.proxy.delete_app,
                           _app.App,
                           method_args=[gateway, app],
                           expected_args=[app],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )


class TestQuota(TestApiGatewayProxy):
    def test_quotas(self):
        gateway = _gateway.Gateway()
        app = _app.App()
        self.verify_get(self.proxy.quotas,
                        _app.Quota,
                        method_args=[gateway, app, ],
                        expected_args=[],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None,
                                         'app_id': None,
                                         'requires_id': False}
                        )


class TestAuth(TestApiGatewayProxy):
    def test_list_api_bound_to_app(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_api_bound_to_app,
                         _auth.ApiAuthInfo,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_list_api_not_bound_to_app(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_apps_bound_to_api,
                         _auth.ApiAuthInfo,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_list_app_bound_to_api(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_api_not_bound_to_app,
                         _auth.ApiAuth,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_create_auth(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.api_auth.'
            'ApiAuthInfo._authorize_apps',
            self.proxy.create_auth_in_api,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )

    def test_delete_auth(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.api_auth.'
            'ApiAuthInfo._cancel_auth',
            self.proxy.delete_auth_from_api,
            method_args=[gateway],
            expected_args=[self.proxy],
            method_kwargs={'auth_id': None},
            expected_kwargs={'app_auth_id': None,
                             'gateway_id': None}
        )


class TestAclPolicy(TestApiGatewayProxy):
    def test_create_acl_policy(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_acl_policy,
                           _ac.AclPolicy,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_update_acl_policy(self):
        gateway = _gateway.Gateway()
        ac = _ac.AclPolicy()
        self.verify_update(self.proxy.update_acl_policy,
                           _ac.AclPolicy,
                           method_args=[gateway, ac],
                           expected_args=[ac],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_delete_acl_policy(self):
        gateway = _gateway.Gateway()
        ac = _ac.AclPolicy()
        self.verify_delete(self.proxy.delete_acl_policy,
                           _ac.AclPolicy,
                           method_args=[gateway, ac],
                           expected_args=[ac],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_list_acl_policies(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.acl_policies,
                         _ac.AclPolicy,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_get_acl_policy(self):
        gateway = _gateway.Gateway()
        ac = _ac.AclPolicy()
        self.verify_get(self.proxy.get_acl_policy,
                        _ac.AclPolicy,
                        method_args=[gateway, ac],
                        expected_args=[ac],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None}
                        )

    def test_delete_acl_policies(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.acl_policy.'
            'AclPolicy._delete_multiple_acls',
            self.proxy.delete_acl_policies,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )


class TestAclPolicyBinding(TestApiGatewayProxy):
    def test_list_apis_for_acl(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_apis_for_acl,
                         _acl_api.ApiForAcl,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_list_api_not_bound_to_acl(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_api_not_bound_to_acl,
                         _acl_api.UnbindApiForAcl,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_list_acl_for_api(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.list_acl_for_api,
                         _acl_api.AclForApi,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_bind_acl_to_api(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.acl_api_binding.'
            'AclApiBinding._bind_to_api',
            self.proxy.bind_acl_to_api,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )

    def test_unbind_acl(self):
        gateway = _gateway.Gateway()
        acl = _acl_api.AclApiBinding()
        self.verify_delete(self.proxy.unbind_acl,
                           _acl_api.AclApiBinding,
                           method_args=[gateway, acl],
                           expected_args=[acl],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_unbind_acls(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.acl_api_binding.'
            'AclBindingFailure._unbind_multiple_acls',
            self.proxy.unbind_acls,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )


class TestCustomAuthorizer(TestApiGatewayProxy):
    def test_list_custom_authorizers(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.custom_authorizers,
                         _custom_auth.CustomAuthorizer,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_get_custom_authorizer(self):
        gateway = _gateway.Gateway()
        custom_auth = _custom_auth.CustomAuthorizer()
        self.verify_get(self.proxy.get_custom_authorizer,
                        _custom_auth.CustomAuthorizer,
                        method_args=[gateway, custom_auth],
                        expected_args=[custom_auth],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None}
                        )

    def test_create_custom_authorizer(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_custom_authorizer,
                           _custom_auth.CustomAuthorizer,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_update_custom_authorizer(self):
        gateway = _gateway.Gateway()
        custom_auth = _custom_auth.CustomAuthorizer()
        self.verify_update(self.proxy.update_custom_authorizer,
                           _custom_auth.CustomAuthorizer,
                           method_args=[gateway, custom_auth],
                           expected_args=[custom_auth],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_delete_custom_authorizer(self):
        gateway = _gateway.Gateway()
        custom_auth = _custom_auth.CustomAuthorizer()
        self.verify_delete(self.proxy.delete_custom_authorizer,
                           _custom_auth.CustomAuthorizer,
                           method_args=[gateway, custom_auth],
                           expected_args=[custom_auth],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )


class TestExportApi(TestApiGatewayProxy):
    def test_import_api(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.export_api.'
            'ImportApi._import_api',
            self.proxy.import_api,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )

    def test_export_api(self):
        gateway = _gateway.Gateway()
        full_path = ''
        self._verify(
            'otcextensions.sdk.apig.v2.export_api.'
            'ExportApi._export_api',
            self.proxy.export_api,
            method_args=[gateway, full_path],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'full_path': full_path}
        )


class TestVpc(TestApiGatewayProxy):
    def test_create_vpc_channel(self):
        gateway = _gateway.Gateway()
        self.verify_create(self.proxy.create_vpc_channel,
                           _vpc.VpcChannel,
                           method_args=[gateway],
                           expected_args=[],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_update_vpc_channel(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self.verify_update(self.proxy.update_vpc_channel,
                           _vpc.VpcChannel,
                           method_args=[gateway, vpc_channel],
                           expected_args=[vpc_channel],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_delete_vpc_channel(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self.verify_delete(self.proxy.delete_vpc_channel,
                           _vpc.VpcChannel,
                           method_args=[gateway, vpc_channel],
                           expected_args=[vpc_channel],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None}
                           )

    def test_get_vpc_channel(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self.verify_get(self.proxy.get_vpc_channel,
                        _vpc.VpcChannel,
                        method_args=[gateway, vpc_channel],
                        expected_args=[vpc_channel],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None}
                        )

    def test_list_vpc_channels(self):
        gateway = _gateway.Gateway()
        self.verify_list(self.proxy.vpc_channels,
                         _vpc.VpcChannel,
                         method_args=[gateway],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None}
                         )

    def test_modify_vpc_channel_healthcheck(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self._verify(
            'otcextensions.sdk.apig.v2.vpc_channel.'
            'VpcChannel.modify_healthcheck',
            self.proxy.modify_vpc_channel_healthcheck,
            method_args=[gateway, vpc_channel],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_channel_id': None}
        )

    def test_add_or_update_backend_server_group(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self._verify(
            'otcextensions.sdk.apig.v2.backend_server_group.'
            'BackendServerGroup.create_group',
            self.proxy.add_or_update_backend_server_group,
            method_args=[gateway, vpc_channel],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_channel_id': None}
        )

    def test_backend_server_groups(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self.verify_list(self.proxy.backend_server_groups,
                         _vpc_sg.BackendServerGroup,
                         method_args=[gateway, vpc_channel],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None,
                                          'vpc_channel_id': None}
                         )

    def test_get_backend_server_group(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server_group = _vpc_sg.BackendServerGroup()
        self.verify_get(self.proxy.get_backend_server_group,
                        _vpc_sg.BackendServerGroup,
                        method_args=[gateway, vpc_channel,
                                     backend_server_group],
                        expected_args=[None],
                        method_kwargs={},
                        expected_kwargs={'gateway_id': None,
                                         'vpc_channel_id': None}
                        )

    def test_update_backend_server_group(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server_group = _vpc_sg.BackendServerGroup()
        self.verify_update(self.proxy.update_backend_server_group,
                           _vpc_sg.BackendServerGroup,
                           method_args=[gateway, vpc_channel,
                                        backend_server_group],
                           expected_args=[None],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None,
                                            'vpc_channel_id': None}
                           )

    def test_delete_backend_server_group(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server_group = _vpc_sg.BackendServerGroup()
        self.verify_delete(self.proxy.delete_backend_server_group,
                           _vpc_sg.BackendServerGroup,
                           method_args=[gateway, vpc_channel,
                                        backend_server_group],
                           expected_args=[None],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None,
                                            'vpc_channel_id': None}
                           )

    def test_add_or_update_backend_servers(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self._verify(
            'otcextensions.sdk.apig.v2.backend_server.'
            'BackendServer.create_members',
            self.proxy.add_or_update_backend_servers,
            method_args=[gateway, vpc_channel],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_channel_id': None}
        )

    def test_list_backend_servers(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self.verify_list(self.proxy.list_backend_servers,
                         _vpc_s.BackendServer,
                         method_args=[gateway, vpc_channel],
                         expected_args=[],
                         method_kwargs={},
                         expected_kwargs={'gateway_id': None,
                                          'vpc_chan_id': None}
                         )

    def test_update_backend_servers(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        self._verify(
            'otcextensions.sdk.apig.v2.backend_server.'
            'BackendServer.update_members',
            self.proxy.update_backend_server,
            method_args=[gateway, vpc_channel],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_channel_id': None}
        )

    def test_delete_backend_servers(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server = _vpc_s.BackendServer()
        self.verify_delete(self.proxy.remove_backend_server,
                           _vpc_s.BackendServer,
                           method_args=[gateway, vpc_channel, backend_server],
                           expected_args=[backend_server],
                           method_kwargs={},
                           expected_kwargs={'gateway_id': None,
                                            'vpc_chan_id': None}
                           )

    def test_enable_backend_server(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server = _vpc_s.BackendServer()
        self._verify(
            'otcextensions.sdk.apig.v2.backend_server.'
            'BackendServer.enable_server',
            self.proxy.enable_backend_server,
            method_args=[gateway, vpc_channel, backend_server],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_chan_id': None}
        )

    def test_disable_backend_server(self):
        gateway = _gateway.Gateway()
        vpc_channel = _vpc.VpcChannel()
        backend_server = _vpc_s.BackendServer()
        self._verify(
            'otcextensions.sdk.apig.v2.backend_server.'
            'BackendServer.disable_server',
            self.proxy.disable_backend_server,
            method_args=[gateway, vpc_channel, backend_server],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None,
                             'vpc_chan_id': None}
        )


class TestApiCall(TestApiGatewayProxy):
    def test_list_api_calls_for_period(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.api_call.'
            'ApiCallResult.get_api_calls_for_period',
            self.proxy.list_api_calls_for_period,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )

    def test_list_api_calls_for_group(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.api_call.'
            'ApiCallResult.get_api_calls_for_group',
            self.proxy.list_api_calls_for_group,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )

    def test_list_metric_data(self):
        gateway = _gateway.Gateway()
        self._verify(
            'otcextensions.sdk.apig.v2.metric_data.'
            'MetricData.get_metric_data',
            self.proxy.list_metric_data,
            method_args=[gateway],
            expected_args=[self.proxy],
            expected_kwargs={'gateway_id': None}
        )
